from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.order_fill_transaction import OrderFillTransaction
    from ..models.take_profit_order_transaction import TakeProfitOrderTransaction
    from ..models.stop_loss_order_transaction import StopLossOrderTransaction
    from ..models.order_cancel_transaction import OrderCancelTransaction
    from ..models.trailing_stop_loss_order_transaction import (
        TrailingStopLossOrderTransaction,
    )


T = TypeVar("T", bound="SetTradeDependentOrdersResponse200")


@_attrs_define
class SetTradeDependentOrdersResponse200:
    """
    Attributes:
        take_profit_order_cancel_transaction (Union[Unset, OrderCancelTransaction]): An OrderCancelTransaction
            represents the cancellation of an Order in the client's Account.
        take_profit_order_transaction (Union[Unset, TakeProfitOrderTransaction]): A TakeProfitOrderTransaction
            represents the creation of a TakeProfit Order in the user's Account.
        take_profit_order_fill_transaction (Union[Unset, OrderFillTransaction]): An OrderFillTransaction represents the
            filling of an Order in the client's Account.
        take_profit_order_created_cancel_transaction (Union[Unset, OrderCancelTransaction]): An OrderCancelTransaction
            represents the cancellation of an Order in the client's Account.
        stop_loss_order_cancel_transaction (Union[Unset, OrderCancelTransaction]): An OrderCancelTransaction represents
            the cancellation of an Order in the client's Account.
        stop_loss_order_transaction (Union[Unset, StopLossOrderTransaction]): A StopLossOrderTransaction represents the
            creation of a StopLoss Order in the user's Account.
        stop_loss_order_fill_transaction (Union[Unset, OrderFillTransaction]): An OrderFillTransaction represents the
            filling of an Order in the client's Account.
        stop_loss_order_created_cancel_transaction (Union[Unset, OrderCancelTransaction]): An OrderCancelTransaction
            represents the cancellation of an Order in the client's Account.
        trailing_stop_loss_order_cancel_transaction (Union[Unset, OrderCancelTransaction]): An OrderCancelTransaction
            represents the cancellation of an Order in the client's Account.
        trailing_stop_loss_order_transaction (Union[Unset, TrailingStopLossOrderTransaction]): A
            TrailingStopLossOrderTransaction represents the creation of a TrailingStopLoss Order in the user's Account.
        related_transaction_i_ds (Union[Unset, List[str]]): The IDs of all Transactions that were created while
            satisfying the request.
        last_transaction_id (Union[Unset, str]): The ID of the most recent Transaction created for the Account
    """

    take_profit_order_cancel_transaction: Union[Unset, "OrderCancelTransaction"] = UNSET
    take_profit_order_transaction: Union[Unset, "TakeProfitOrderTransaction"] = UNSET
    take_profit_order_fill_transaction: Union[Unset, "OrderFillTransaction"] = UNSET
    take_profit_order_created_cancel_transaction: Union[
        Unset, "OrderCancelTransaction"
    ] = UNSET
    stop_loss_order_cancel_transaction: Union[Unset, "OrderCancelTransaction"] = UNSET
    stop_loss_order_transaction: Union[Unset, "StopLossOrderTransaction"] = UNSET
    stop_loss_order_fill_transaction: Union[Unset, "OrderFillTransaction"] = UNSET
    stop_loss_order_created_cancel_transaction: Union[
        Unset, "OrderCancelTransaction"
    ] = UNSET
    trailing_stop_loss_order_cancel_transaction: Union[
        Unset, "OrderCancelTransaction"
    ] = UNSET
    trailing_stop_loss_order_transaction: Union[
        Unset, "TrailingStopLossOrderTransaction"
    ] = UNSET
    related_transaction_i_ds: Union[Unset, List[str]] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        take_profit_order_cancel_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_order_cancel_transaction, Unset):
            take_profit_order_cancel_transaction = (
                self.take_profit_order_cancel_transaction.to_dict()
            )

        take_profit_order_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_order_transaction, Unset):
            take_profit_order_transaction = self.take_profit_order_transaction.to_dict()

        take_profit_order_fill_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit_order_fill_transaction, Unset):
            take_profit_order_fill_transaction = (
                self.take_profit_order_fill_transaction.to_dict()
            )

        take_profit_order_created_cancel_transaction: Union[Unset, Dict[str, Any]] = (
            UNSET
        )
        if not isinstance(self.take_profit_order_created_cancel_transaction, Unset):
            take_profit_order_created_cancel_transaction = (
                self.take_profit_order_created_cancel_transaction.to_dict()
            )

        stop_loss_order_cancel_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_order_cancel_transaction, Unset):
            stop_loss_order_cancel_transaction = (
                self.stop_loss_order_cancel_transaction.to_dict()
            )

        stop_loss_order_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_order_transaction, Unset):
            stop_loss_order_transaction = self.stop_loss_order_transaction.to_dict()

        stop_loss_order_fill_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_order_fill_transaction, Unset):
            stop_loss_order_fill_transaction = (
                self.stop_loss_order_fill_transaction.to_dict()
            )

        stop_loss_order_created_cancel_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss_order_created_cancel_transaction, Unset):
            stop_loss_order_created_cancel_transaction = (
                self.stop_loss_order_created_cancel_transaction.to_dict()
            )

        trailing_stop_loss_order_cancel_transaction: Union[Unset, Dict[str, Any]] = (
            UNSET
        )
        if not isinstance(self.trailing_stop_loss_order_cancel_transaction, Unset):
            trailing_stop_loss_order_cancel_transaction = (
                self.trailing_stop_loss_order_cancel_transaction.to_dict()
            )

        trailing_stop_loss_order_transaction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trailing_stop_loss_order_transaction, Unset):
            trailing_stop_loss_order_transaction = (
                self.trailing_stop_loss_order_transaction.to_dict()
            )

        related_transaction_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.related_transaction_i_ds, Unset):
            related_transaction_i_ds = self.related_transaction_i_ds

        last_transaction_id = self.last_transaction_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if take_profit_order_cancel_transaction is not UNSET:
            field_dict["takeProfitOrderCancelTransaction"] = (
                take_profit_order_cancel_transaction
            )
        if take_profit_order_transaction is not UNSET:
            field_dict["takeProfitOrderTransaction"] = take_profit_order_transaction
        if take_profit_order_fill_transaction is not UNSET:
            field_dict["takeProfitOrderFillTransaction"] = (
                take_profit_order_fill_transaction
            )
        if take_profit_order_created_cancel_transaction is not UNSET:
            field_dict["takeProfitOrderCreatedCancelTransaction"] = (
                take_profit_order_created_cancel_transaction
            )
        if stop_loss_order_cancel_transaction is not UNSET:
            field_dict["stopLossOrderCancelTransaction"] = (
                stop_loss_order_cancel_transaction
            )
        if stop_loss_order_transaction is not UNSET:
            field_dict["stopLossOrderTransaction"] = stop_loss_order_transaction
        if stop_loss_order_fill_transaction is not UNSET:
            field_dict["stopLossOrderFillTransaction"] = (
                stop_loss_order_fill_transaction
            )
        if stop_loss_order_created_cancel_transaction is not UNSET:
            field_dict["stopLossOrderCreatedCancelTransaction"] = (
                stop_loss_order_created_cancel_transaction
            )
        if trailing_stop_loss_order_cancel_transaction is not UNSET:
            field_dict["trailingStopLossOrderCancelTransaction"] = (
                trailing_stop_loss_order_cancel_transaction
            )
        if trailing_stop_loss_order_transaction is not UNSET:
            field_dict["trailingStopLossOrderTransaction"] = (
                trailing_stop_loss_order_transaction
            )
        if related_transaction_i_ds is not UNSET:
            field_dict["relatedTransactionIDs"] = related_transaction_i_ds
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.order_fill_transaction import OrderFillTransaction
        from ..models.take_profit_order_transaction import TakeProfitOrderTransaction
        from ..models.stop_loss_order_transaction import StopLossOrderTransaction
        from ..models.order_cancel_transaction import OrderCancelTransaction
        from ..models.trailing_stop_loss_order_transaction import (
            TrailingStopLossOrderTransaction,
        )

        d = src_dict.copy()
        _take_profit_order_cancel_transaction = d.pop(
            "takeProfitOrderCancelTransaction", UNSET
        )
        take_profit_order_cancel_transaction: Union[Unset, OrderCancelTransaction]
        if isinstance(_take_profit_order_cancel_transaction, Unset):
            take_profit_order_cancel_transaction = UNSET
        else:
            take_profit_order_cancel_transaction = OrderCancelTransaction.from_dict(
                _take_profit_order_cancel_transaction
            )

        _take_profit_order_transaction = d.pop("takeProfitOrderTransaction", UNSET)
        take_profit_order_transaction: Union[Unset, TakeProfitOrderTransaction]
        if isinstance(_take_profit_order_transaction, Unset):
            take_profit_order_transaction = UNSET
        else:
            take_profit_order_transaction = TakeProfitOrderTransaction.from_dict(
                _take_profit_order_transaction
            )

        _take_profit_order_fill_transaction = d.pop(
            "takeProfitOrderFillTransaction", UNSET
        )
        take_profit_order_fill_transaction: Union[Unset, OrderFillTransaction]
        if isinstance(_take_profit_order_fill_transaction, Unset):
            take_profit_order_fill_transaction = UNSET
        else:
            take_profit_order_fill_transaction = OrderFillTransaction.from_dict(
                _take_profit_order_fill_transaction
            )

        _take_profit_order_created_cancel_transaction = d.pop(
            "takeProfitOrderCreatedCancelTransaction", UNSET
        )
        take_profit_order_created_cancel_transaction: Union[
            Unset, OrderCancelTransaction
        ]
        if isinstance(_take_profit_order_created_cancel_transaction, Unset):
            take_profit_order_created_cancel_transaction = UNSET
        else:
            take_profit_order_created_cancel_transaction = (
                OrderCancelTransaction.from_dict(
                    _take_profit_order_created_cancel_transaction
                )
            )

        _stop_loss_order_cancel_transaction = d.pop(
            "stopLossOrderCancelTransaction", UNSET
        )
        stop_loss_order_cancel_transaction: Union[Unset, OrderCancelTransaction]
        if isinstance(_stop_loss_order_cancel_transaction, Unset):
            stop_loss_order_cancel_transaction = UNSET
        else:
            stop_loss_order_cancel_transaction = OrderCancelTransaction.from_dict(
                _stop_loss_order_cancel_transaction
            )

        _stop_loss_order_transaction = d.pop("stopLossOrderTransaction", UNSET)
        stop_loss_order_transaction: Union[Unset, StopLossOrderTransaction]
        if isinstance(_stop_loss_order_transaction, Unset):
            stop_loss_order_transaction = UNSET
        else:
            stop_loss_order_transaction = StopLossOrderTransaction.from_dict(
                _stop_loss_order_transaction
            )

        _stop_loss_order_fill_transaction = d.pop("stopLossOrderFillTransaction", UNSET)
        stop_loss_order_fill_transaction: Union[Unset, OrderFillTransaction]
        if isinstance(_stop_loss_order_fill_transaction, Unset):
            stop_loss_order_fill_transaction = UNSET
        else:
            stop_loss_order_fill_transaction = OrderFillTransaction.from_dict(
                _stop_loss_order_fill_transaction
            )

        _stop_loss_order_created_cancel_transaction = d.pop(
            "stopLossOrderCreatedCancelTransaction", UNSET
        )
        stop_loss_order_created_cancel_transaction: Union[Unset, OrderCancelTransaction]
        if isinstance(_stop_loss_order_created_cancel_transaction, Unset):
            stop_loss_order_created_cancel_transaction = UNSET
        else:
            stop_loss_order_created_cancel_transaction = (
                OrderCancelTransaction.from_dict(
                    _stop_loss_order_created_cancel_transaction
                )
            )

        _trailing_stop_loss_order_cancel_transaction = d.pop(
            "trailingStopLossOrderCancelTransaction", UNSET
        )
        trailing_stop_loss_order_cancel_transaction: Union[
            Unset, OrderCancelTransaction
        ]
        if isinstance(_trailing_stop_loss_order_cancel_transaction, Unset):
            trailing_stop_loss_order_cancel_transaction = UNSET
        else:
            trailing_stop_loss_order_cancel_transaction = (
                OrderCancelTransaction.from_dict(
                    _trailing_stop_loss_order_cancel_transaction
                )
            )

        _trailing_stop_loss_order_transaction = d.pop(
            "trailingStopLossOrderTransaction", UNSET
        )
        trailing_stop_loss_order_transaction: Union[
            Unset, TrailingStopLossOrderTransaction
        ]
        if isinstance(_trailing_stop_loss_order_transaction, Unset):
            trailing_stop_loss_order_transaction = UNSET
        else:
            trailing_stop_loss_order_transaction = (
                TrailingStopLossOrderTransaction.from_dict(
                    _trailing_stop_loss_order_transaction
                )
            )

        related_transaction_i_ds = cast(
            List[str], d.pop("relatedTransactionIDs", UNSET)
        )

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        set_trade_dependent_orders_response_200 = cls(
            take_profit_order_cancel_transaction=take_profit_order_cancel_transaction,
            take_profit_order_transaction=take_profit_order_transaction,
            take_profit_order_fill_transaction=take_profit_order_fill_transaction,
            take_profit_order_created_cancel_transaction=take_profit_order_created_cancel_transaction,
            stop_loss_order_cancel_transaction=stop_loss_order_cancel_transaction,
            stop_loss_order_transaction=stop_loss_order_transaction,
            stop_loss_order_fill_transaction=stop_loss_order_fill_transaction,
            stop_loss_order_created_cancel_transaction=stop_loss_order_created_cancel_transaction,
            trailing_stop_loss_order_cancel_transaction=trailing_stop_loss_order_cancel_transaction,
            trailing_stop_loss_order_transaction=trailing_stop_loss_order_transaction,
            related_transaction_i_ds=related_transaction_i_ds,
            last_transaction_id=last_transaction_id,
        )

        set_trade_dependent_orders_response_200.additional_properties = d
        return set_trade_dependent_orders_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.trailing_stop_loss_details import TrailingStopLossDetails
    from ..models.stop_loss_details import StopLossDetails
    from ..models.take_profit_details import TakeProfitDetails


T = TypeVar("T", bound="SetTradeDependentOrdersBody")


@_attrs_define
class SetTradeDependentOrdersBody:
    """
    Attributes:
        take_profit (Union[Unset, TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Take
            Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss (Union[Unset, StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to be
            created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop Loss,
            or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss (Union[Unset, TrailingStopLossDetails]): TrailingStopLossDetails specifies the details of a
            Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is filled that opens
            a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order is modified
            directly through the Trade.
    """

    take_profit: Union[Unset, "TakeProfitDetails"] = UNSET
    stop_loss: Union[Unset, "StopLossDetails"] = UNSET
    trailing_stop_loss: Union[Unset, "TrailingStopLossDetails"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        take_profit: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.take_profit, Unset):
            take_profit = self.take_profit.to_dict()

        stop_loss: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stop_loss, Unset):
            stop_loss = self.stop_loss.to_dict()

        trailing_stop_loss: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trailing_stop_loss, Unset):
            trailing_stop_loss = self.trailing_stop_loss.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if take_profit is not UNSET:
            field_dict["takeProfit"] = take_profit
        if stop_loss is not UNSET:
            field_dict["stopLoss"] = stop_loss
        if trailing_stop_loss is not UNSET:
            field_dict["trailingStopLoss"] = trailing_stop_loss

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.trailing_stop_loss_details import TrailingStopLossDetails
        from ..models.stop_loss_details import StopLossDetails
        from ..models.take_profit_details import TakeProfitDetails

        d = src_dict.copy()
        _take_profit = d.pop("takeProfit", UNSET)
        take_profit: Union[Unset, TakeProfitDetails]
        if isinstance(_take_profit, Unset):
            take_profit = UNSET
        else:
            take_profit = TakeProfitDetails.from_dict(_take_profit)

        _stop_loss = d.pop("stopLoss", UNSET)
        stop_loss: Union[Unset, StopLossDetails]
        if isinstance(_stop_loss, Unset):
            stop_loss = UNSET
        else:
            stop_loss = StopLossDetails.from_dict(_stop_loss)

        _trailing_stop_loss = d.pop("trailingStopLoss", UNSET)
        trailing_stop_loss: Union[Unset, TrailingStopLossDetails]
        if isinstance(_trailing_stop_loss, Unset):
            trailing_stop_loss = UNSET
        else:
            trailing_stop_loss = TrailingStopLossDetails.from_dict(_trailing_stop_loss)

        set_trade_dependent_orders_body = cls(
            take_profit=take_profit,
            stop_loss=stop_loss,
            trailing_stop_loss=trailing_stop_loss,
        )

        set_trade_dependent_orders_body.additional_properties = d
        return set_trade_dependent_orders_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.take_profit_details_time_in_force import (
    check_take_profit_details_time_in_force,
)
from ..models.take_profit_details_time_in_force import TakeProfitDetailsTimeInForce
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="TakeProfitDetails")


@_attrs_define
class TakeProfitDetails:
    """TakeProfitDetails specifies the details of a Take Profit Order to be created on behalf of a client. This may happen
    when an Order is filled that opens a Trade requiring a Take Profit, or when a Trade's dependent Take Profit Order is
    modified directly through the Trade.

        Attributes:
            price (Union[Unset, str]): The price that the Take Profit Order will be triggered at. Only one of the price and
                distance fields may be specified.
            time_in_force (Union[Unset, TakeProfitDetailsTimeInForce]): The time in force for the created Take Profit Order.
                This may only be GTC, GTD or GFD.
            gtd_time (Union[Unset, str]): The date when the Take Profit Order will be cancelled on if timeInForce is GTD.
            client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
    """

    price: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, TakeProfitDetailsTimeInForce] = UNSET
    gtd_time: Union[Unset, str] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        price = self.price

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        gtd_time = self.gtd_time

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if price is not UNSET:
            field_dict["price"] = price
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if gtd_time is not UNSET:
            field_dict["gtdTime"] = gtd_time
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        price = d.pop("price", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, TakeProfitDetailsTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_take_profit_details_time_in_force(_time_in_force)

        gtd_time = d.pop("gtdTime", UNSET)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        take_profit_details = cls(
            price=price,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            client_extensions=client_extensions,
        )

        take_profit_details.additional_properties = d
        return take_profit_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="UnitsAvailableDetails")


@_attrs_define
class UnitsAvailableDetails:
    """Representation of many units of an Instrument are available to be traded for both long and short Orders.

    Attributes:
        long (Union[Unset, str]): The units available for long Orders.
        short (Union[Unset, str]): The units available for short Orders.
    """

    long: Union[Unset, str] = UNSET
    short: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        long = self.long

        short = self.short

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if long is not UNSET:
            field_dict["long"] = long
        if short is not UNSET:
            field_dict["short"] = short

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        long = d.pop("long", UNSET)

        short = d.pop("short", UNSET)

        units_available_details = cls(
            long=long,
            short=short,
        )

        units_available_details.additional_properties = d
        return units_available_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="LiquidityRegenerationScheduleStep")


@_attrs_define
class LiquidityRegenerationScheduleStep:
    """A liquidity regeneration schedule Step indicates the amount of bid and ask liquidity that is used by the Account at
    a certain time. These amounts will only change at the timestamp of the following step.

        Attributes:
            timestamp (Union[Unset, str]): The timestamp of the schedule step.
            bid_liquidity_used (Union[Unset, str]): The amount of bid liquidity used at this step in the schedule.
            ask_liquidity_used (Union[Unset, str]): The amount of ask liquidity used at this step in the schedule.
    """

    timestamp: Union[Unset, str] = UNSET
    bid_liquidity_used: Union[Unset, str] = UNSET
    ask_liquidity_used: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        timestamp = self.timestamp

        bid_liquidity_used = self.bid_liquidity_used

        ask_liquidity_used = self.ask_liquidity_used

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if bid_liquidity_used is not UNSET:
            field_dict["bidLiquidityUsed"] = bid_liquidity_used
        if ask_liquidity_used is not UNSET:
            field_dict["askLiquidityUsed"] = ask_liquidity_used

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        timestamp = d.pop("timestamp", UNSET)

        bid_liquidity_used = d.pop("bidLiquidityUsed", UNSET)

        ask_liquidity_used = d.pop("askLiquidityUsed", UNSET)

        liquidity_regeneration_schedule_step = cls(
            timestamp=timestamp,
            bid_liquidity_used=bid_liquidity_used,
            ask_liquidity_used=ask_liquidity_used,
        )

        liquidity_regeneration_schedule_step.additional_properties = d
        return liquidity_regeneration_schedule_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

FixedPriceOrderTransactionReason = Literal["PLATFORM_ACCOUNT_MIGRATION"]

FIXED_PRICE_ORDER_TRANSACTION_REASON_VALUES: Set[FixedPriceOrderTransactionReason] = {
    "PLATFORM_ACCOUNT_MIGRATION",
}


def check_fixed_price_order_transaction_reason(
    value: str,
) -> FixedPriceOrderTransactionReason:
    if value in FIXED_PRICE_ORDER_TRANSACTION_REASON_VALUES:
        return cast(FixedPriceOrderTransactionReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {FIXED_PRICE_ORDER_TRANSACTION_REASON_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.order_client_extensions_modify_transaction_type import (
    check_order_client_extensions_modify_transaction_type,
)
from ..models.order_client_extensions_modify_transaction_type import (
    OrderClientExtensionsModifyTransactionType,
)
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="OrderClientExtensionsModifyTransaction")


@_attrs_define
class OrderClientExtensionsModifyTransaction:
    """A OrderClientExtensionsModifyTransaction represents the modification of an Order's Client Extensions.

    Attributes:
        id (Union[Unset, str]): The Transaction's Identifier.
        time (Union[Unset, str]): The date/time when the Transaction was created.
        user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
        batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
        type (Union[Unset, OrderClientExtensionsModifyTransactionType]): The Type of the Transaction. Always set to
            "ORDER_CLIENT_EXTENSIONS_MODIFY" for a OrderClienteExtensionsModifyTransaction.
        order_id (Union[Unset, str]): The ID of the Order who's client extensions are to be modified.
        client_order_id (Union[Unset, str]): The original Client ID of the Order who's client extensions are to be
            modified.
        client_extensions_modify (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        trade_client_extensions_modify (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to
            attach a clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this
            field if your account is associated with MT4.
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    type: Union[Unset, OrderClientExtensionsModifyTransactionType] = UNSET
    order_id: Union[Unset, str] = UNSET
    client_order_id: Union[Unset, str] = UNSET
    client_extensions_modify: Union[Unset, "ClientExtensions"] = UNSET
    trade_client_extensions_modify: Union[Unset, "ClientExtensions"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        order_id = self.order_id

        client_order_id = self.client_order_id

        client_extensions_modify: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions_modify, Unset):
            client_extensions_modify = self.client_extensions_modify.to_dict()

        trade_client_extensions_modify: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trade_client_extensions_modify, Unset):
            trade_client_extensions_modify = (
                self.trade_client_extensions_modify.to_dict()
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id
        if type is not UNSET:
            field_dict["type"] = type
        if order_id is not UNSET:
            field_dict["orderID"] = order_id
        if client_order_id is not UNSET:
            field_dict["clientOrderID"] = client_order_id
        if client_extensions_modify is not UNSET:
            field_dict["clientExtensionsModify"] = client_extensions_modify
        if trade_client_extensions_modify is not UNSET:
            field_dict["tradeClientExtensionsModify"] = trade_client_extensions_modify

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, OrderClientExtensionsModifyTransactionType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_order_client_extensions_modify_transaction_type(_type)

        order_id = d.pop("orderID", UNSET)

        client_order_id = d.pop("clientOrderID", UNSET)

        _client_extensions_modify = d.pop("clientExtensionsModify", UNSET)
        client_extensions_modify: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions_modify, Unset):
            client_extensions_modify = UNSET
        else:
            client_extensions_modify = ClientExtensions.from_dict(
                _client_extensions_modify
            )

        _trade_client_extensions_modify = d.pop("tradeClientExtensionsModify", UNSET)
        trade_client_extensions_modify: Union[Unset, ClientExtensions]
        if isinstance(_trade_client_extensions_modify, Unset):
            trade_client_extensions_modify = UNSET
        else:
            trade_client_extensions_modify = ClientExtensions.from_dict(
                _trade_client_extensions_modify
            )

        order_client_extensions_modify_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            order_id=order_id,
            client_order_id=client_order_id,
            client_extensions_modify=client_extensions_modify,
            trade_client_extensions_modify=trade_client_extensions_modify,
        )

        order_client_extensions_modify_transaction.additional_properties = d
        return order_client_extensions_modify_transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.close_transaction_type import check_close_transaction_type
from ..models.close_transaction_type import CloseTransactionType
from typing import Union


T = TypeVar("T", bound="CloseTransaction")


@_attrs_define
class CloseTransaction:
    """A CloseTransaction represents the closing of an Account.

    Attributes:
        id (Union[Unset, str]): The Transaction's Identifier.
        time (Union[Unset, str]): The date/time when the Transaction was created.
        user_id (Union[Unset, int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Union[Unset, str]): The ID of the Account the Transaction was created for.
        batch_id (Union[Unset, str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Union[Unset, str]): The Request ID of the request which generated the transaction.
        type (Union[Unset, CloseTransactionType]): The Type of the Transaction. Always set to "CLOSE" in a
            CloseTransaction.
    """

    id: Union[Unset, str] = UNSET
    time: Union[Unset, str] = UNSET
    user_id: Union[Unset, int] = UNSET
    account_id: Union[Unset, str] = UNSET
    batch_id: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    type: Union[Unset, CloseTransactionType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        time = self.time

        user_id = self.user_id

        account_id = self.account_id

        batch_id = self.batch_id

        request_id = self.request_id

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if time is not UNSET:
            field_dict["time"] = time
        if user_id is not UNSET:
            field_dict["userID"] = user_id
        if account_id is not UNSET:
            field_dict["accountID"] = account_id
        if batch_id is not UNSET:
            field_dict["batchID"] = batch_id
        if request_id is not UNSET:
            field_dict["requestID"] = request_id
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        time = d.pop("time", UNSET)

        user_id = d.pop("userID", UNSET)

        account_id = d.pop("accountID", UNSET)

        batch_id = d.pop("batchID", UNSET)

        request_id = d.pop("requestID", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, CloseTransactionType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_close_transaction_type(_type)

        close_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
        )

        close_transaction.additional_properties = d
        return close_transaction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.order import Order


T = TypeVar("T", bound="ListOrdersResponse200")


@_attrs_define
class ListOrdersResponse200:
    """
    Attributes:
        orders (Union[Unset, List['Order']]): The list of Order detail objects
        last_transaction_id (Union[Unset, str]): The ID of the most recent Transaction created for the Account
    """

    orders: Union[Unset, List["Order"]] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        orders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.orders, Unset):
            orders = []
            for orders_item_data in self.orders:
                orders_item = orders_item_data.to_dict()
                orders.append(orders_item)

        last_transaction_id = self.last_transaction_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if orders is not UNSET:
            field_dict["orders"] = orders
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.order import Order

        d = src_dict.copy()
        orders = []
        _orders = d.pop("orders", UNSET)
        for orders_item_data in _orders or []:
            orders_item = Order.from_dict(orders_item_data)

            orders.append(orders_item)

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        list_orders_response_200 = cls(
            orders=orders,
            last_transaction_id=last_transaction_id,
        )

        list_orders_response_200.additional_properties = d
        return list_orders_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.trailing_stop_loss_order_state import check_trailing_stop_loss_order_state
from ..models.trailing_stop_loss_order_state import TrailingStopLossOrderState
from ..models.trailing_stop_loss_order_time_in_force import (
    check_trailing_stop_loss_order_time_in_force,
)
from ..models.trailing_stop_loss_order_time_in_force import (
    TrailingStopLossOrderTimeInForce,
)
from ..models.trailing_stop_loss_order_trigger_condition import (
    check_trailing_stop_loss_order_trigger_condition,
)
from ..models.trailing_stop_loss_order_trigger_condition import (
    TrailingStopLossOrderTriggerCondition,
)
from ..models.trailing_stop_loss_order_type import check_trailing_stop_loss_order_type
from ..models.trailing_stop_loss_order_type import TrailingStopLossOrderType
from typing import cast
from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="TrailingStopLossOrder")


@_attrs_define
class TrailingStopLossOrder:
    """A TrailingStopLossOrder is an order that is linked to an open Trade and created with a price distance. The price
    distance is used to calculate a trailing stop value for the order that is in the losing direction from the market
    price at the time of the order's creation. The trailing stop value will follow the market price as it moves in the
    winning direction, and the order will filled (closing the Trade) by the first price that is equal to or worse than
    the trailing stop value. A TrailingStopLossOrder cannot be used to open a new Position.

        Attributes:
            id (Union[Unset, str]): The Order's identifier, unique within the Order's Account.
            create_time (Union[Unset, str]): The time when the Order was created.
            state (Union[Unset, TrailingStopLossOrderState]): The current state of the Order.
            client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            type (Union[Unset, TrailingStopLossOrderType]): The type of the Order. Always set to "TRAILING_STOP_LOSS" for
                Trailing Stop Loss Orders.
            trade_id (Union[Unset, str]): The ID of the Trade to close when the price threshold is breached.
            client_trade_id (Union[Unset, str]): The client ID of the Trade to be closed when the price threshold is
                breached.
            distance (Union[Unset, str]): The price distance (in price units) specified for the TrailingStopLoss Order.
            time_in_force (Union[Unset, TrailingStopLossOrderTimeInForce]): The time-in-force requested for the
                TrailingStopLoss Order. Restricted to "GTC", "GFD" and "GTD" for TrailingStopLoss Orders.
            gtd_time (Union[Unset, str]): The date/time when the StopLoss Order will be cancelled if its timeInForce is
                "GTD".
            trigger_condition (Union[Unset, TrailingStopLossOrderTriggerCondition]): Specification of which price component
                should be used when determining if an Order should be triggered and filled. This allows Orders to be triggered
                based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy) price
                depending on the desired behaviour. Orders are always filled using their default price component.
                This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
                condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
                transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
                is set to the default value when indicating the distance from an Order's trigger price, and will always provide
                the default trigger condition when creating or modifying an Order.
                A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
                value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
                a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
            trailing_stop_value (Union[Unset, str]): The trigger price for the Trailing Stop Loss Order. The trailing stop
                value will trail (follow) the market price by the TSL order's configured "distance" as the market price moves in
                the winning direction. If the market price moves to a level that is equal to or worse than the trailing stop
                value, the order will be filled and the Trade will be closed.
            filling_transaction_id (Union[Unset, str]): ID of the Transaction that filled this Order (only provided when the
                Order's state is FILLED)
            filled_time (Union[Unset, str]): Date/time when the Order was filled (only provided when the Order's state is
                FILLED)
            trade_opened_id (Union[Unset, str]): Trade ID of Trade opened when the Order was filled (only provided when the
                Order's state is FILLED and a Trade was opened as a result of the fill)
            trade_reduced_id (Union[Unset, str]): Trade ID of Trade reduced when the Order was filled (only provided when
                the Order's state is FILLED and a Trade was reduced as a result of the fill)
            trade_closed_i_ds (Union[Unset, List[str]]): Trade IDs of Trades closed when the Order was filled (only provided
                when the Order's state is FILLED and one or more Trades were closed as a result of the fill)
            cancelling_transaction_id (Union[Unset, str]): ID of the Transaction that cancelled the Order (only provided
                when the Order's state is CANCELLED)
            cancelled_time (Union[Unset, str]): Date/time when the Order was cancelled (only provided when the state of the
                Order is CANCELLED)
            replaces_order_id (Union[Unset, str]): The ID of the Order that was replaced by this Order (only provided if
                this Order was created as part of a cancel/replace).
            replaced_by_order_id (Union[Unset, str]): The ID of the Order that replaced this Order (only provided if this
                Order was cancelled as part of a cancel/replace).
    """

    id: Union[Unset, str] = UNSET
    create_time: Union[Unset, str] = UNSET
    state: Union[Unset, TrailingStopLossOrderState] = UNSET
    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    type: Union[Unset, TrailingStopLossOrderType] = UNSET
    trade_id: Union[Unset, str] = UNSET
    client_trade_id: Union[Unset, str] = UNSET
    distance: Union[Unset, str] = UNSET
    time_in_force: Union[Unset, TrailingStopLossOrderTimeInForce] = UNSET
    gtd_time: Union[Unset, str] = UNSET
    trigger_condition: Union[Unset, TrailingStopLossOrderTriggerCondition] = UNSET
    trailing_stop_value: Union[Unset, str] = UNSET
    filling_transaction_id: Union[Unset, str] = UNSET
    filled_time: Union[Unset, str] = UNSET
    trade_opened_id: Union[Unset, str] = UNSET
    trade_reduced_id: Union[Unset, str] = UNSET
    trade_closed_i_ds: Union[Unset, List[str]] = UNSET
    cancelling_transaction_id: Union[Unset, str] = UNSET
    cancelled_time: Union[Unset, str] = UNSET
    replaces_order_id: Union[Unset, str] = UNSET
    replaced_by_order_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        create_time = self.create_time

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state

        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        trade_id = self.trade_id

        client_trade_id = self.client_trade_id

        distance = self.distance

        time_in_force: Union[Unset, str] = UNSET
        if not isinstance(self.time_in_force, Unset):
            time_in_force = self.time_in_force

        gtd_time = self.gtd_time

        trigger_condition: Union[Unset, str] = UNSET
        if not isinstance(self.trigger_condition, Unset):
            trigger_condition = self.trigger_condition

        trailing_stop_value = self.trailing_stop_value

        filling_transaction_id = self.filling_transaction_id

        filled_time = self.filled_time

        trade_opened_id = self.trade_opened_id

        trade_reduced_id = self.trade_reduced_id

        trade_closed_i_ds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.trade_closed_i_ds, Unset):
            trade_closed_i_ds = self.trade_closed_i_ds

        cancelling_transaction_id = self.cancelling_transaction_id

        cancelled_time = self.cancelled_time

        replaces_order_id = self.replaces_order_id

        replaced_by_order_id = self.replaced_by_order_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if create_time is not UNSET:
            field_dict["createTime"] = create_time
        if state is not UNSET:
            field_dict["state"] = state
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions
        if type is not UNSET:
            field_dict["type"] = type
        if trade_id is not UNSET:
            field_dict["tradeID"] = trade_id
        if client_trade_id is not UNSET:
            field_dict["clientTradeID"] = client_trade_id
        if distance is not UNSET:
            field_dict["distance"] = distance
        if time_in_force is not UNSET:
            field_dict["timeInForce"] = time_in_force
        if gtd_time is not UNSET:
            field_dict["gtdTime"] = gtd_time
        if trigger_condition is not UNSET:
            field_dict["triggerCondition"] = trigger_condition
        if trailing_stop_value is not UNSET:
            field_dict["trailingStopValue"] = trailing_stop_value
        if filling_transaction_id is not UNSET:
            field_dict["fillingTransactionID"] = filling_transaction_id
        if filled_time is not UNSET:
            field_dict["filledTime"] = filled_time
        if trade_opened_id is not UNSET:
            field_dict["tradeOpenedID"] = trade_opened_id
        if trade_reduced_id is not UNSET:
            field_dict["tradeReducedID"] = trade_reduced_id
        if trade_closed_i_ds is not UNSET:
            field_dict["tradeClosedIDs"] = trade_closed_i_ds
        if cancelling_transaction_id is not UNSET:
            field_dict["cancellingTransactionID"] = cancelling_transaction_id
        if cancelled_time is not UNSET:
            field_dict["cancelledTime"] = cancelled_time
        if replaces_order_id is not UNSET:
            field_dict["replacesOrderID"] = replaces_order_id
        if replaced_by_order_id is not UNSET:
            field_dict["replacedByOrderID"] = replaced_by_order_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        create_time = d.pop("createTime", UNSET)

        _state = d.pop("state", UNSET)
        state: Union[Unset, TrailingStopLossOrderState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = check_trailing_stop_loss_order_state(_state)

        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        _type = d.pop("type", UNSET)
        type: Union[Unset, TrailingStopLossOrderType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = check_trailing_stop_loss_order_type(_type)

        trade_id = d.pop("tradeID", UNSET)

        client_trade_id = d.pop("clientTradeID", UNSET)

        distance = d.pop("distance", UNSET)

        _time_in_force = d.pop("timeInForce", UNSET)
        time_in_force: Union[Unset, TrailingStopLossOrderTimeInForce]
        if isinstance(_time_in_force, Unset):
            time_in_force = UNSET
        else:
            time_in_force = check_trailing_stop_loss_order_time_in_force(_time_in_force)

        gtd_time = d.pop("gtdTime", UNSET)

        _trigger_condition = d.pop("triggerCondition", UNSET)
        trigger_condition: Union[Unset, TrailingStopLossOrderTriggerCondition]
        if isinstance(_trigger_condition, Unset):
            trigger_condition = UNSET
        else:
            trigger_condition = check_trailing_stop_loss_order_trigger_condition(
                _trigger_condition
            )

        trailing_stop_value = d.pop("trailingStopValue", UNSET)

        filling_transaction_id = d.pop("fillingTransactionID", UNSET)

        filled_time = d.pop("filledTime", UNSET)

        trade_opened_id = d.pop("tradeOpenedID", UNSET)

        trade_reduced_id = d.pop("tradeReducedID", UNSET)

        trade_closed_i_ds = cast(List[str], d.pop("tradeClosedIDs", UNSET))

        cancelling_transaction_id = d.pop("cancellingTransactionID", UNSET)

        cancelled_time = d.pop("cancelledTime", UNSET)

        replaces_order_id = d.pop("replacesOrderID", UNSET)

        replaced_by_order_id = d.pop("replacedByOrderID", UNSET)

        trailing_stop_loss_order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
            type=type,
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            distance=distance,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            trigger_condition=trigger_condition,
            trailing_stop_value=trailing_stop_value,
            filling_transaction_id=filling_transaction_id,
            filled_time=filled_time,
            trade_opened_id=trade_opened_id,
            trade_reduced_id=trade_reduced_id,
            trade_closed_i_ds=trade_closed_i_ds,
            cancelling_transaction_id=cancelling_transaction_id,
            cancelled_time=cancelled_time,
            replaces_order_id=replaces_order_id,
            replaced_by_order_id=replaced_by_order_id,
        )

        trailing_stop_loss_order.additional_properties = d
        return trailing_stop_loss_order

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, Set, cast

Direction = Literal["LONG", "SHORT"]

DIRECTION_VALUES: Set[Direction] = {
    "LONG",
    "SHORT",
}


def check_direction(value: str) -> Direction:
    if value in DIRECTION_VALUES:
        return cast(Direction, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {DIRECTION_VALUES!r}")



from typing import Literal, Set, cast

TradePL = Literal["NEGATIVE", "POSITIVE", "ZERO"]

TRADE_PL_VALUES: Set[TradePL] = {
    "NEGATIVE",
    "POSITIVE",
    "ZERO",
}


def check_trade_pl(value: str) -> TradePL:
    if value in TRADE_PL_VALUES:
        return cast(TradePL, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {TRADE_PL_VALUES!r}")



from typing import Literal, Set, cast

TakeProfitOrderState = Literal["CANCELLED", "FILLED", "PENDING", "TRIGGERED"]

TAKE_PROFIT_ORDER_STATE_VALUES: Set[TakeProfitOrderState] = {
    "CANCELLED",
    "FILLED",
    "PENDING",
    "TRIGGERED",
}


def check_take_profit_order_state(value: str) -> TakeProfitOrderState:
    if value in TAKE_PROFIT_ORDER_STATE_VALUES:
        return cast(TakeProfitOrderState, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {TAKE_PROFIT_ORDER_STATE_VALUES!r}"
    )
